# Working with Animations
You can use CocosBuilder for creating character animations, animating complete scenes or just about any animation you can imagine. The animation editor has full support for multiple resolutions, easing between keyframes, boned animations and multiple timelines to name a few of the features.

## The Basics
In the bottom of the main window you can find the timeline. You use the timeline to create your animations.

![image](6-1.png?raw=true)

By default your ccb-file has a single timeline that is 10 seconds long. CocosBuilder edits animations at a frame rate of 30 frames per second, but when you play back the animation in your app it will use whatever you have set cocos2d to use (the default in cocos2d is 60 fps). The current time is displayed in the top right corner, and has the format minute:second:frame. The blue vertical line also shows the current time. Click the time display to change the duration of the current timeline.

### Adding Keyframes
Animations in CocosBuilder are keyframe based. You can add keyframes to different properties of a node and CocosBuilder will automatically interpolate between the keyframes, optionally with different types of easing.

To add a keyframe, first expand the view of the node by clicking the triangle to the right of the name of the node. This will reveal all the animatable properties of the node. What can be animated varies slightly depending on what type of node you have selected. Once the properties are visible you can click the property in the timeline with the option key held down. This will create a new keyframe at the time of the click. Alternatively, you can create a new keyframe at the time of the time marker by selecting a node then choosing *Insert Keyframe* in the *Animation* menu.

Keyframes are automatically added at the current time if you transform a node in the canvas area, given that the transformed property already has one or more keyframes in the timeline.

### Editing Keyframes
You edit a specific keyframe of a node by moving the time marker to the time of the keyframe and selecting the node. You can focus on a keyframe by double clicking it (which will select the node and move the time marker).

You can select keyframes and move them together by dragging a selection box around them. You can also copy and paste keyframes between nodes. Make sure you only have one selected node when pasting the keyframes. The keyframes will be pasted starting at the time of the time marker.

If you have selected a set of keyframes it is possible to reverse the order of them by selecting *Reverse Selected Keyframes* in the *Animation* menu. Use the *Stretch Selected Keyframes…* option to speed up or slow down an animation by a scaling factor.

### Importing a Sequence of Images
If you have an animation created by sprite frames it can be tedious to move each individual frame to the timeline. CocosBuilder simplifies this process by automatically importing a sequence of images. Select the frames that you want to import in the left hand project view, then select a CCSprite in the timeline. Now choose *Create Frames from Selected Resources* in the *Animation* menu. The frames will automatically be created at the start of the marker. If you need to slow down the animation, select the newly created keyframes and use the *Stretch Selected Keyframes…* command.

### Applying Easing
CocosBuilder offers a carefully selected subset of the easings provided by cocos2d. To apply an easing right click between two keyframes and select the type of easing that you want to apply.

![image](6-2.png?raw=true)

Some of the easings have additional options, after the easing has been applied you can right click again and select *Easing Setting…* from the popup menu.

## Using Multiple Timelines
A very powerful feature of CocosBuilder's animation editor is the ability to have multiple timelines in a single file. You can name the different sequences and play them back from your code by using their name. It's even possible to smoothly transition between the different timelines.

To select, add or edit your timelines use the timeline popup menu:

![image](6-3.png?raw=true)

In the edit timelines dialog you can get an overview of your timelines, rename them, add new ones and (optionally) set one of the timelines to automatically start playback directly when the ccbi-file is loaded by your app.

![image](6-4.png?raw=true)

Properties in timelines that do not have keyframes set share their values across timelines. E.g. if you move one node in one timeline it will be moved in all timelines as long as they do not have a keyframe set for the position property. It can sometimes be useful to add a single keyframe to a property just to override the shared value for a specific timeline.

### Chaining Timelines
You can automatically play back a sequence of timelines by chaining them. You can also use this feature for automatically looping a timeline.

![image](6-5.png?raw=true)

To have a timeline play in sequence, click the *No chained timeline* text and select the timeline you want to play right after the current one.

### Playing Back Animations in Code
To programmatically control the animations you create with CocosBuilder you will need to retrieve the *CCBAnimationManager*. The animation manager will be assigned to the nodes *userObject* when the ccbi-file is loaded.

    CCNode* myNodeGraph = [CCBReader nodeGraphFromFile:@"myFile.ccbi"];
    CCBAnimationManager* animationManager = myNodeGraph.userObject;

The animation manager will be returned as an autoreleased object. To play back a specific timeline call the *runAnimationsForSequenceNamed:* method. If a timeline is currently playing it will be immediately stopped when calling this method.

    [animationManager runAnimationsForSequenceNamed:@"My Timeline"];

Optionally, you can use a tween duration to smoothly transition to the new timeline. Where possible linear interpolations will be used for the transition.

    [animationManager runAnimationsForSequenceNamed:@"My Timeline" tweenDuration:0.5f];

It is also possible to receive a callback whenever a timeline has finished playing. You will receive these callbacks even if another timeline is chained in sequence. Use the CCBAnimationManagerDelegate to receive the callbacks.
